/*!
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */body{background-color:#fff;min-height:100vh;overflow-y:scroll;padding:70px 0;position:relative}nav{box-shadow:0 3px 3px #aaa;z-index:999}pre{overflow:auto;word-wrap:normal}pre,pre code{white-space:pre}pre code{overflow-wrap:normal}input,select{margin:0}div.container{padding-left:15px;padding-right:15px;width:98%}.navbar a{color:#51504f;font-weight:700}.navbar a .caret{color:#e2d2e2}.navbar li.dropdown .dropdown-menu{max-height:85vh;overflow-y:auto}.navbar-nav li.dropdown:focus-within>.dropdown-menu,.navbar-nav li.dropdown:hover>.dropdown-menu{display:block}.navbar-nav li.dropdown:hover>.dropdown-menu{z-index:1001}.navbar-brand{align-items:center;display:inline-flex;height:60px;padding:0 15px}.brand-logo{height:40px;overflow:visible!important;width:104px}@keyframes pinSpin{0%{transform:rotate(0) translateX(0)}to{transform:rotate(1turn) translateX(0)}}@media (prefers-reduced-motion:no-preference){.navbar-brand:hover .brand-logo-pinwheel{animation:pinSpin 1.5s linear;transform-origin:17.66px 17.66px}}.navbar-user-icon{align-items:center;background-color:#c0defb;border-radius:50%;color:#017cee;display:inline-flex;height:30px;justify-content:center;margin:-5px 0;vertical-align:middle;width:30px}span.status_square{border:1px solid grey;cursor:pointer;display:inline-block;height:10px;padding-left:0;width:10px}div.squares{float:right;font-size:1}.d3-tip{background:#000000d9;border:0;border-radius:5px;color:#fff;margin-top:-4px;padding:10px}input#execution_date{margin-bottom:0;width:220px}table.highlighttable{table-layout:fixed;width:100%}div.linenodiv{padding-right:1px!important}.linenos{border:0;width:50px}div.linenodiv pre{background-color:initial;color:#8e8e8d;padding-left:0;padding-right:4px;text-align:right}.wrap{white-space:pre-wrap}.code{font-family:monospace}#sql{border:1px solid #ccc;border-radius:5px}.ace_editor div{font:inherit!important}#ace_container{margin:10px 0}#sql_ace{visibility:hidden}table.dataframe{font-size:12px}table.dataframe tbody tr td{padding:2px}table.dataframe.dataTable thead>tr>th{padding:10px 20px 10px 10px}table.dataTable.dataframe thead .sorting{background:url(f7490d556a6c42e49ba4.png) no-repeat 100%}table.dataTable.dataframe thead .sorting_desc{background:url(649c0b07771e68fafdeb.png) no-repeat 100%}table.dataTable.dataframe thead .sorting_asc{background:url(48f0ea180c40270a5b05.png) no-repeat 100%}.no-wrap{white-space:nowrap}div.form-inline{margin-bottom:5px}body div.panel{padding:0}.legend-row{align-items:center;display:flex;justify-content:space-between}.legend-item{align-items:center;border:2px solid #cbcbcb;border-radius:4px;display:inline-flex;font-size:11px;margin-right:4px;padding:0 3px}.legend-item--interactive{cursor:pointer}.legend-item--no-border{border-color:#0000;padding:0}.legend-item:last-child{margin-right:0}.legend-item__swatch{background:#fff;border:1px solid #8e8e8d;border-radius:2px;display:inline-block;height:12px;margin-right:3px;width:12px}.legend-item__swatch--circle{border-radius:50%;height:14px;width:14px}.legend-item__swatch--no-border{border-color:#0000}#timezone-other,label[for=timezone-other]{font-weight:400;margin:3px 20px}#timezone-menu{overflow-x:hidden}#timezone-menu ul.typeahead.dropdown-menu{border:none;left:0!important;max-height:200px;overflow-x:hidden;overflow-y:auto}#timezone-menu .dropdown-item{padding:3px 20px}.ui-menu .ui-menu-item a.ui-state-active,.ui-menu .ui-menu-item a.ui-state-focus{background:#f6f6f6;border:1px solid #fff;border-left-color:#ccc;color:#454545}.accordion-toggle,.bootstrap-datetimepicker-widget .datepicker>div{display:block}.accordion-toggle.collapsed>.toggle-direction{transform:rotate(180deg)}.hll{background-color:#ffc}.c{color:#408080;font-style:italic}.err{border:1px solid red}.k{color:green;font-weight:700}.o{color:#666}.cm{color:#408080;font-style:italic}.cp{color:#bc7a00}.c1,.cs{color:#408080;font-style:italic}.gd{color:#a00000}.ge{font-style:italic}.gr{color:red}.gh{color:navy;font-weight:700}.gi{color:#00a000}.go{color:#888}.gp{color:navy}.gp,.gs,.gu{font-weight:700}.gu{color:purple}.gt{color:#04d}.kc,.kd,.kn{color:green;font-weight:700}.kp,.kr{color:green}.kr{font-weight:700}.kt{color:#b00040}.m{color:#666}.s{color:#ba2121}.na{color:#7d9029}.nb{color:green}.nc{color:#00f;font-weight:700}.no{color:#800}.nd{color:#a2f}.ni{color:#999}.ne,.ni{font-weight:700}.ne{color:#d2413a}.nf{color:#00f}.nl{color:#a0a000}.nn{color:#00f}.nn,.nt{font-weight:700}.nt{color:green}.nv{color:#19177c}.ow{color:#a2f;font-weight:700}.w{color:#bbb}.mb,.mf,.mh,.mi,.mo{color:#666}.sb,.sc,.sd{color:#ba2121}.sd{font-style:italic}.s1,.s2{color:#ba2121}.footer{align-items:center;background-color:#f5f5f5;bottom:0;color:#8e8e8d;display:flex;height:60px;justify-content:space-between;position:absolute;width:100%}.dag-view-tools{background-color:#f0f0f0;margin:16px 0;padding:16px 0}.code-wrap{margin-top:30px;position:relative}.code-wrap-toggle{position:absolute;right:15px;top:15px}.search-input{position:relative}.search-input__input{padding-right:40px}.search-input__clear-btn{border:0;position:absolute;right:2px;top:2px}.btn-group{display:inline-flex}.panel-body .panel-group+div{overflow-x:auto}.task-instance-modal-column{margin-top:8px;overflow-x:auto}.refresh-actions{align-items:center;display:inline-flex;float:right;justify-content:flex-end;margin-bottom:15px;min-width:225px;position:relative;right:10px}.refresh-actions>.switch-label{margin:0 10px}.loading-dots.refresh-loading{display:none}.trigger-dropdown-btn{border-bottom-right-radius:0;border-right-width:0;border-top-right-radius:0}.trigger-dropdown-menu{left:-112px}.dropdown-form-btn{background-color:#fff;border:none;color:#51504f;line-height:1.428571429;padding:3px 20px;text-align:left;width:100%}.dropdown-form-btn:focus,.dropdown-form-btn:hover{background-color:#f5f5f5;color:#262626}.tooltip{max-width:300px;z-index:0}.tooltip.d3-tip,.tooltip.in{z-index:1070}details summary{display:list-item}.menu-scroll{max-height:300px;overflow-y:auto}.next-asset-triggered:hover{background-color:#cbcbcb;cursor:pointer}